



#include <iostream>
#include <vector>

#include "ploughshare/ploughshare.h"
#include "appl_grid/appl_grid.h"

#include "LHAPDF/LHAPDF.h"

extern "C" void evolvepdf_(const double& x, const double& Q, double* xf); 
extern "C" double alphaspdf_(const double& Q);


void grid_diff( appl::grid& g, TH1D* h ) { 

  TH1D*   href = g.getReference();
  double  evnt = g.run(); 
  
  for ( int i=1 ; i<=href->GetNbinsX() ; i++ ) { 

    double ref = href->GetBinContent(i)/evnt;

    std::cout << "i=" << i << "\t" << ref << " " << h->GetBinContent(i) << "\t(" << h->GetBinContent(i)/ref << ")" << std::endl;
  }


}


template<typename T>
inline std::ostream& operator<<( std::ostream& s, const std::vector<T>& v ) { 
  for ( size_t i=0 ; i<v.size() ; i++ ) s << "\n\t" << v[i];
  return s;
} 


template<typename T>
inline std::ostream& operator<<( std::ostream& s, const std::set<T>& v ) { 
  for ( typename std::set<T>::const_iterator it=v.begin() ; it!=v.end() ; it++ ) s << "\n\t" << *it;
  return s;
} 



template<typename T, typename U>
inline std::ostream& operator<<( std::ostream& s, const std::map<T,U>& v ) { 
  for ( typename std::map<T,U>::const_iterator  it=v.begin() ; it!=v.end() ; it++ ) s << "\n\t" << it->first << "\n\t" << it->second;
  return s;
} 


template<typename T, typename U>
inline std::ostream& operator<<( std::ostream& s, const std::map<T,const U>& v ) { 
  for ( typename std::map<T,const U>::const_iterator  it=v.begin() ; it!=v.end() ; it++ ) s << "\n\t" << it->first << "\t" << it->second;
  return s;
} 


int main( int argc, char** argv ) { 
  
  std::string newdataset = "";
  std::string newpdf     = "CT10";

  bool custompdf = false;

  for ( int i=1 ; i<argc ; i++ ) {
 
    std::string arg = argv[i];

    if ( arg == "-d" ) { 
      if ( ++i<argc ) {
	newdataset = argv[i];
      }
      else { 
	std::cerr << "no dataset specified" << std::endl;
	return -1;
      }
    }

    if ( arg == "-p" ) { 
      if ( ++i<argc ) {
	custompdf = true;
	newpdf = argv[i];
      }
      else { 
	std::cerr << "no pdf specified" << std::endl;
	return -1;
      }
    }
  }


  ploughshare p; 
  
  std::cout << p.path() << std::endl;
  
  // p.verbose( true );
  
  if ( newdataset!="" ) p.fetch( newdataset );
  p.fetch( "atlas-atlas-wpm-arxiv-1109.5141" );
  p.fetch( "atlas-atlas-z0-arxiv-1109.5141" );
  p.fetch( "fastnlo-h1-jets-arxiv-1406.4709-xsec000.tab.gz" );
  p.fetch( "atlas-atlas-wpm-arxiv-1612.03016" );
  p.fetch( "atlas-atlas-dijets-arxiv-1312.3524" );
  p.fetch( "atlas-atlas-dijets-arxiv-1112.6297" );


  std::cout << "dataset path:  " << p.path("atlas-atlas-wpm-arxiv-1612.03016")  << std::endl;
  std::cout << "dataset grids: " << p.grids("atlas-atlas-wpm-arxiv-1612.03016") << std::endl;

  std::cout << std::endl;



  std::cout << "dataset:       " << p.dataset("atlas-atlas-z0-arxiv-1109.5141")  << std::endl;

  std::cout << "dataset path:  " << p.dataset("atlas-atlas-z0-arxiv-1109.5141").path()  << std::endl;

  std::cout << "dataset grids: " << p.dataset("atlas-atlas-z0-arxiv-1109.5141").grids() << std::endl;



  ploughshare::dataset_t  ds = p.dataset("atlas-atlas-dijets-arxiv-1312.3524");
  std::cout << "dataset path:  " << ds.path() << std::endl;
  std::cout << "dataset grids: " << ds.grids() << std::endl;

  std::cout << std::endl;

  //  std::vector<std::string> grids = p.grids("atlas-atlas-dijets-arxiv-1312.3524");
  std::vector<std::string> grids = p.grids("atlas-atlas-z0-arxiv-1109.5141");


  std::string _pdfname = newpdf;

  int  ipdf = 0;

  for ( size_t i=0 ; i<grids.size() ; i++ ) { 
    
    appl::grid g(grids[i]);

    /// before convoluting, need to set up some pdf routine
    //    g.convolute( ... );


    if ( !custompdf ) { 
      if (  g.getGeneratedPDF()!="" ) {
	std::cout << "generated PDF: " << g.getGeneratedPDF() << std::endl;	  
	LHAPDF::initPDFSet( g.getGeneratedPDF().c_str(), g.getGeneratediPDF() );
      }
      else if ( newpdf == "" ) { 
	std::cerr << "no pdf specified" << std::endl;
	return -1;
      }
      else {
	std::cout << "using defaul pdf: " << _pdfname << std::endl;
	LHAPDF::initPDFSet( _pdfname, 0 );
      }
    }
    else { 
      LHAPDF::initPDFSet( _pdfname, ipdf );
    }

    TH1D* h = g.convolute( evolvepdf_, alphaspdf_ );

    std::vector<double> xsec = g.vconvolute( evolvepdf_, alphaspdf_ );

    std::cout << xsec << std::endl;

    grid_diff( g, h );

  }


  return 0;
}

